#define M_PI 3.14159265358979323846

#include <cmath>
#include <iostream>

using namespace std;

void cubicSolve(double a, double b, double c, double d, double* roots)
{
    double p = (3 * a * c - b * b) / (3 * a * a);
    double q = (2 * b*b*b - 9 * a*b*c + 27 * a*a*d) / (27 * a*a*a);
    double discrim = (q * q) / 4 + (p * p * p) / 27;
    double x0 = - b / a / 3;
    double third = 1.0 / 3;
    double x1_real = 0.0;
    double x1_imag = 0.0;
    double x2_real = 0.0;
    double x2_imag = 0.0;
    double x3_real = 0.0;
    double x3_imag = 0.0;
    if (discrim > 0.0)
    {
        double s = - q / 2 + sqrt(discrim);
        s = ((s < 0) ? -pow(-s, third) : pow(s, third));
        double t = - q / 2 - sqrt(discrim);
        t = ((t < 0) ? -pow(-t, third) : pow(t, third));
        roots[0] = x0 + s + t;
        roots[1] = 0.0;
        roots[2] = x0 - (s + t) / 2;
        roots[3] = sqrt(3.0) * (s - t) / 2;
        roots[4] = roots[2];
        roots[5] = -roots[3];
    }
    else if (discrim < 0.0)
    {
        double r = sqrt(- p*p*p / 27);
        double phi = acos(-q / 2 / r);
        double rho = 2 * pow(r, third);
        roots[0] = x0 + rho * cos(phi / 3);
        roots[1] = 0.0;
        roots[2] = x0 + rho * cos((phi + 2.0 * M_PI) / 3.0);
        roots[3] = 0.0;
        roots[4] = x0 + rho * cos((phi + 4.0 * M_PI) / 3.0);
        roots[5] = 0.0;
    }
    else
    {
        double s = (q < 0) ? pow(-q / 2, third) : -pow(q / 2, third);
        roots[0] = x0 + 2 * s;
        roots[1] = 0.0;
        roots[2] = x0 - s;
        roots[3] = 0.0;
        roots[4] = roots[2];
        roots[5] = 0.0;
    }
}


double getTime() 
{
    const long COUNT = 10000;
    double x[COUNT * 3];
    double r[COUNT * 6];
    srand(time(NULL));
    for (long i = 0; i < COUNT * 3; i++) {
        x[i] = rand() * 2;
    }
    clock_t start_time, end_time;
    start_time = clock();
    double roots[6];
    for (long j = 0; j < COUNT; j++) {
        cubicSolve(1.0, x[j*3], x[j*3+1], x[j*3+2], roots);
        for (long i = 0; i < 6; i++) {
            r[j*6+i] = roots[i];
        }
    }
    end_time = clock();
    return (double)(end_time - start_time) / CLOCKS_PER_SEC;
}

int main()
{
    // double t = getTime();
    // cout << "Time to solve the cubic equations: " << t * 1000 << " ms" << endl;
    double roots[6];
    cubicSolve(1., 200., 8000., 2., roots);
    for (int i = 0; i < 6; i++) 
    {
        cout << roots[i] << endl;
    }
}
