import math

def cubic_solve(a0, b0, c0, d0):
    ''' Reduce the cubic equation to to form:
        x**3 + a*x**2 + b*x + c = 0 '''
    a, b, c = b0 / a0, c0 / a0, d0 / a0

    # Some repeating constants and variables
    third = 1. / 3.
    a13 = a * third
    a2 = a13 * a13
    sqr3 = math.sqrt(3)

    # Additional intermediate variables
    f = third * b - a2
    g = a13 * (2 * a2 - b) + c
    h = 0.25 * g * g + f * f * f

    def cubic_root(x):
        ''' Compute cubic root of a number while maintaining its sign'''
        if x.real >= 0:
            return x**third
        else:
            return -(-x)**third

    if f == g == h == 0:
        r1 = -cubic_root(c)
        return r1, r1, r1

    elif h <= 0:
        j = math.sqrt(-f)
        k = math.acos(-0.5 * g / (j * j * j))
        m = math.cos(third * k)
        n = sqr3 * math.sin(third * k)
        r1 = 2*j*m - a13
        r2 = -j * (m + n) - a13
        r3 = -j * (m - n) - a13
        return r1, r2, r3

    else:
        sqrt_h = math.sqrt(h)
        S = cubic_root(-0.5 * g + sqrt_h)
        U = cubic_root(-0.5 * g - sqrt_h)
        S_plus_U = S + U
        S_minus_U = S - U
        r1 = S_plus_U - a13
        r2 = -0.5 * S_plus_U - a13 + S_minus_U * sqr3 * 0.5j
        r3 = -0.5 * S_plus_U - a13 - S_minus_U * sqr3 * 0.5j
        return r1, r2, r3