#include <iostream>
#include <Eigen/Dense>
/*
NOTE: Download `Eigen` module from https://eigen.tuxfamily.org/dox/index.html .
      Specify the include path when compiling.
*/

using namespace std;
 
void cubicSolve(double a, double b, double c, double d, double *roots)
{
    b = b / a;
    c = c / a;
    d = d / a;
    Eigen::Matrix3d m;
    m << 0, 0, -d,
         1, 0, -c,
         0, 1, -b;
    Eigen::Vector3cd es = m.eigenvalues();
    roots[0] = es[0].real();
    roots[1] = es[0].imag();
    roots[2] = es[0].real();
    roots[3] = es[0].imag();
    roots[4] = es[0].real();
    roots[5] = es[0].imag();
}

double getTime()
{
    const long COUNT = 10000;
    double x[COUNT * 3];
    double r[COUNT * 6];
    srand(time(NULL));
    for (long i = 0; i < COUNT * 3; i++) {
        x[i] = rand() * 2;
    }
    clock_t start_time, end_time;
    start_time = clock();
    double roots[6];
    for (long j = 0; j < COUNT; j++) {
        cubicSolve(1.0, x[j*3], x[j*3+1], x[j*3+2], roots);
        for (long i = 0; i < 6; i++) {
            r[j*6+i] = roots[i];
        }
    }
    end_time = clock();
    return (double)(end_time - start_time) / CLOCKS_PER_SEC;
}

int main()
{
    double t = getTime();
    cout << "Time to solve the cubic equations: " << t * 1000 << " ms" << endl;
    return 0;
}