#include <iostream>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>
#include <vector>

void cubicSolve(double a, double b, double c, double d, double* roots);

namespace py = pybind11;
using namespace std;

vector<double> cubicSolveSingle(double a, double b, double c, double d)
{
    double roots[6];
    cubicSolve(a, b, c, d, roots);
    vector<double> result(roots, roots + 6);
    return result;
}

vector<vector<double>> cubicSolveMultiple(vector<vector<double>> &&coeffs)
{
    size_t size = coeffs.size();
    vector<vector<double>> result;
    result.reserve(size);
    double roots[6];
    for (vector<double> & c : coeffs)
    {
        cubicSolve(c[0], c[1], c[2], c[3], roots);
        vector<double> roots_vec(roots, roots + 6);
        result.push_back(roots_vec);
    }
    return result;
}

PYBIND11_MODULE(mycubicmodule, m) {
    m.doc() = "";
    m.def("cubic_solve", &cubicSolveSingle, "");
    m.def("cubic_solve_multi", &cubicSolveMultiple, "");
}
